/** 
 *  Error Simulation
 */
ABB.Simulator.Tools.ErrorSimulation = function() {
	/* Private Variables */
	var objSD = null;
	var objFS = null;
	var _nominalCurrent = 200;										// Nominal current value for phase currents
	var _nominalEarthCurrent = 50;									// Nominal current value for earth fault current
	var _injectedValue = new Array (0, 0, 0, 0, 10);			// injected currents and voltage via sliders: L1, L2, L3, I0, U0
	var _isLightSensorEnabled = new Array(false, false, false); 	// Is the light sensor enabled?
	var _arcOn = new Array ('107298336', '107297312', '107296288');
	var _arcTreshold = new Array ( ['107298080', '107298016'], ['107297056', '107296992'], ['107296032', '107295968']);	
	var _overcurrentStartFlags =  new Array ( [false, false, false], [false, false, false], [false, false, false], [false, false, false] );	//I>: L1, L2, L3, I1>>: L1, L2, L3, I2>>: L1, L2, L3, I>>>: L1, L2, L3
	var _overcurrentTripFlags = new Array ( [false, false, false], [false, false, false], [false, false, false], [false, false, false] );		// Contains boolean value if trip condition is met or not
	var _earthFaultStartFlags = new Array (false, false, false);
	var _earthFaultTripFlags = new Array (false, false, false);
	var _earthDirection = new Array ('107802992', '107796400', '107267552'); //L1 L2 H
	var _minVoltage = new Array ('107798832', '107792240', '107805424');
	var _phaseStartFlag = false;
	var _phaseTripFlag = false;
	var _negativeStartFlag = new Array(false, false);
	var _negativeTripFlag = new Array(false, false);
	var _signalTreshold = new Array('107218784','107225568','107222176', '107226784', '107271072', '107803440', '107796848', '107268000', '107271968', '107278752', '107275360'); 
	var _delayTime = new Array('107218400','107225184','107221792','107226528', '107270688', '107801136', '107794544', '107266528', '107271840', '107278368', '107274976'); 
	var _NoStartPh = new Array('107218912','107225696','107222304','107225696'); //number of start phases to activate
	var _inputTag = new Array('L1', 'L2', 'L3', 'I0', 'U0');
	var _protectStages = new Array('PHLPTOC1', 'PHHPTOC1', 'PHHPTOC2', 'PHIPTOC1', 'EFHPTOC1', 'DEFLPDEF1', 'DEFLPDEF2', 'DEFHPDEF1', 'PDNSPTOC1', 'NSPTOC1', 'NSPTOC2');
	var _startCounter = new Array(0,0,0,0,0,0);
	var _tripCounter = 0;	
	var _arcFlag = new Array(false, false, false);
	var _arcTripFlag = new Array(false, false, false);
	var _arcMode = new Array("107297696", "107296672", "107295648");
	var _eventList = new Array(50);
	var _eventCounter = 0;
	
	var objLEDs = new ABB.Simulator.UI.LEDs();
	
	/* Private Functions */
	function simulateFaults() {
	
		if(!ABB.Simulator.Global.Ready)return;
		
		var oFSP = ABB.Simulator.Global.GetFaultSimulationPanel();
		var locks = oFSP.getLocks();
		for(var i=0; i<_signalTreshold.length; i++) {
			switch(i) {
				case 0:
					if ( ABB.Simulator.UI.CardMode.getValueNode(107219232)*1 == 0 ) {
						if (!locks[0] ) checkOvercurrentStart(i, 'L1');
						if (!locks[1]) checkOvercurrentStart(i, 'L2');
						if (!locks[2]) checkOvercurrentStart(i, 'L3');
					}
					break;
				case 1:
					if( ABB.Simulator.UI.CardMode.getValueNode(107226016)*1 == 0 ) {
						if (!locks[0]) checkOvercurrentStart(i, 'L1');
						if (!locks[1]) checkOvercurrentStart(i, 'L2');
						if (!locks[2]) checkOvercurrentStart(i, 'L3');
					}
					break;
				case 2:
					if( ABB.Simulator.UI.CardMode.getValueNode(107222624)*1 == 0 ) {
						if (!locks[0]) checkOvercurrentStart(i, 'L1');
						if (!locks[1]) checkOvercurrentStart(i, 'L2');
						if (!locks[2]) checkOvercurrentStart(i, 'L3');
					}
					break;
				case 3:
					if( ABB.Simulator.UI.CardMode.getValueNode(107227232)*1 == 0 ) {
						if (!locks[0]) checkOvercurrentStart(i, 'L1');
						if (!locks[1]) checkOvercurrentStart(i, 'L2');
						if (!locks[2]) checkOvercurrentStart(i, 'L3');
					}
					break;
				case 4:
					if( ABB.Simulator.UI.CardMode.getValueNode(107271200)*1 == 0 ) {
						if (!locks[3]) checkEarthStart(i, 'I0', false);
					}
					break;
				case 5:						
					if ( ABB.Simulator.UI.CardMode.getValueNode(107803568)*1 == 0 ) {
						if (!locks[3]) checkEarthStart(i, 'I0', true);
					}
					break;
				case 6: 
					if ( ABB.Simulator.UI.CardMode.getValueNode(107796976)*1 == 0 ) {
						if (!locks[3]) checkEarthStart(i, 'I0', true);
					}
					break;
				case 7:
					if ( ABB.Simulator.UI.CardMode.getValueNode(107268128)*1 == 0 ) {
						if (!locks[3]) checkEarthStart(i, 'I0', true);
					}
					break;
				case 8:
					if( ABB.Simulator.UI.CardMode.getValueNode(107272096)*1 == 0 ) {						
						var enabled = new Array (false,false,false);
						var counter = 0;
						
						for(var j=0; j<3; j++) {
							if (!locks[j]) {
								enabled[j] = true;
								counter++;
							}
						}
						if (counter >= 2) checkPhaseDiscontinuityStart( i, enabled );												
					}
					break;
				case 9:
					if ( ABB.Simulator.UI.CardMode.getValueNode(107278880)*1 == 0 ) {
						if ( !locks[0] && !locks[1] && !locks[2] ) checkNegativeStart(i, 0);
					}
					break;
				case 10:
					if ( ABB.Simulator.UI.CardMode.getValueNode(107275488)*1 == 0 ) {
						if ( !locks[0] && !locks[1] && !locks[2] ) checkNegativeStart(i, 1);
					}
					break;
			}
		}
	
		var active = new Array (false,false,false);
		for(var i=0; i<4; i++) {
			if (!locks[i]) {
				active[i] = true;
			}
		}
		for(var i=0; i<_isLightSensorEnabled.length; i++) {			
			if(ABB.Simulator.UI.CardMode.getValueNode(_arcOn[i])*1 == 0) {
				if( _isLightSensorEnabled[i] ) {
					checkArcProtectionFault(i, active);
				}
			}
		}				
	}
	
	function shouldLedsBeOff() {
		var signal_active = false;
		
		//_overcurrentStartFlags.length
		for(var i=0; i<_overcurrentStartFlags.length; i++) {
			for(var j=0; j<_overcurrentStartFlags[i].length; j++) {
				if(_overcurrentStartFlags[i][j]) {
					signal_active = true;
				}
			}
		}		
		//_earthFaultStartFlags
		for(var i=0; i<_earthFaultStartFlags.length; i++) {
			if(_earthFaultStartFlags.length[i]) {
				signal_active = true;
			}
		}		
		//_phaseStartFlag
		if(_phaseStartFlag) {
			signal_active = true;
		}
		//_negativeStartFlag
		for(var i=0; i<_negativeStartFlag.length; i++) {
			if(_negativeStartFlag.length[i]) {
				signal_active = true;
			}
		}
		//_arcFlag
		for(var i=0; i<_arcFlag.length; i++) {
			if(_arcFlag[i]) {
				signal_active = true;
			}
		}
		if(!signal_active) {
			objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
		}
		else {
			objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );
		}
	}
		
	/**
	* Checks if selected signal has either overcurrent fault or earth fault
	* Parameters:
	* index			Index from _nodeIs array to get correct node id
	* calculateOvercurrent	Checks for overcurrent fault if true and earth fault if false
	*/
	function checkOvercurrentStart( index, tag ) {
		// get protect stage multiplier
		var val = ABB.Simulator.UI.CardMode.getValueNode(_signalTreshold[index]);		
		for (var i=0; i<_inputTag.length; i++) {
			if ( tag == _inputTag[i] ) {						
				if((val * _nominalCurrent) < _injectedValue[i]) {						
					_overcurrentStartFlags[index][i] = true;						
					ABB.Simulator.Global.isStart = true;						
					if( !ABB.Simulator.Global.breakerOpen ) {					
						objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );						
					}
					waitIfTrip(index, i, true);
					return;							
				}
				else {
					if( _overcurrentStartFlags[index][i] == true && ABB.Simulator.Global.isAlarm == false) {
						_overcurrentStartFlags[index][i] = false;
						ABB.Simulator.Global.isStart = false;
						objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
						objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
						shouldLedsBeOff();
					}
				}
			}
		}													
	}

	function checkEarthStart( index, tag, directional ) {
		// get protect stage multiplier
		var val = ABB.Simulator.UI.CardMode.getValueNode(_signalTreshold[index]);
		if( !directional ) {
			for (var i=0; i<_inputTag.length; i++) {
				if ( tag == _inputTag[i] ) {				
					if((val * _nominalEarthCurrent) < _injectedValue[i]) {
						_earthFaultStartFlags[index-4] = true;
						ABB.Simulator.Global.isStart = true;						
						if( !ABB.Simulator.Global.breakerOpen ) {
							objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );							
						}
					waitIfTrip(index, i, false);
					return;
					}
					else {
						if(_earthFaultStartFlags[index-4] == true && ABB.Simulator.Global.isAlarm == false) {					
							_earthFaultStartFlags[index-4] = false
							ABB.Simulator.Global.isStart = false;
							objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
							objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
							shouldLedsBeOff();
						}
					}
				}			
			}
		}
		else {
			var direction = ABB.Simulator.UI.CardMode.getValueNode(_earthDirection[index-5])*1;
			var voltage = ABB.Simulator.UI.CardMode.getValueNode(_minVoltage[index-5]);
			switch (direction) {
				case 0: //non-directional
					for (var i=0; i<_inputTag.length; i++) {
						if ( tag == _inputTag[i] ) {				
							if( ((val * _nominalEarthCurrent) < _injectedValue[i]) && (voltage < _injectedValue[4]*1000) ) {
								_earthFaultStartFlags[index-4] = true;
								ABB.Simulator.Global.isStart = true;								
								if( !ABB.Simulator.Global.breakerOpen ) {
									objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );									
								}
							waitIfTrip(index, i, false);
							return;
							}
							else {
								if(_earthFaultStartFlags[index-4] == true && ABB.Simulator.Global.isAlarm == false) {					
									_earthFaultStartFlags[index-4] = false
									ABB.Simulator.Global.isStart = false;
									objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
									objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
									shouldLedsBeOff();
								}
							}
						}			
					}
					break;
				case 1: //forward
					for (var i=0; i<_inputTag.length; i++) {
						if ( tag == _inputTag[i] ) {				
							if( ((val * _nominalEarthCurrent) < _injectedValue[i]) && (voltage < _injectedValue[4]*1000) && !ABB.Simulator.Global.currentUp ) {
								_earthFaultStartFlags[index-4] = true;
								ABB.Simulator.Global.isStart = true;								
								if( !ABB.Simulator.Global.breakerOpen ) {
									objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );									
								}
							waitIfTrip(index, i, false);
							return;
							}
							else {
								if(_earthFaultStartFlags[index-4] == true && ABB.Simulator.Global.isAlarm == false) {					
									_earthFaultStartFlags[index-4] = false
									ABB.Simulator.Global.isStart = false;
									objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
									objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
									shouldLedsBeOff();
								}
							}
						}			
					}
					break;
				case 2: //reverse
					for (var i=0; i<_inputTag.length; i++) {
						if ( tag == _inputTag[i] ) {				
							if( ((val * _nominalEarthCurrent) < _injectedValue[i]) && (voltage < _injectedValue[4]*1000) && ABB.Simulator.Global.currentUp ) {
								_earthFaultStartFlags[index-4] = true;
								ABB.Simulator.Global.isStart = true;								
								if( !ABB.Simulator.Global.breakerOpen ) {
									objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );									
								}
							waitIfTrip(index, i, false);
							return;
							}
							else {
								if(_earthFaultStartFlags[index-4] == true && ABB.Simulator.Global.isAlarm == false) {					
									_earthFaultStartFlags[index-4] = false
									ABB.Simulator.Global.isStart = false;
									objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
									objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
									shouldLedsBeOff();
								}
							}
						}			
					}
					break;
			}		
		}				
	}
	
	function checkPhaseDiscontinuityStart( index, enabled ) {		
		var val = ABB.Simulator.UI.CardMode.getValueNode(_signalTreshold[index]) / 100;
		var min = ABB.Simulator.UI.CardMode.getValueNode(107271584)*_nominalCurrent;
		var currents = new Array();
		var Imax;
		var Imin;
		var bigger = false;
		var j=0;
				
		for(var i=0; i<3; i++) {
			if ( enabled[i] ) {
				currents[j] = _injectedValue[i];
				j++;
			}
		}		
		
		Imax = Math.max.apply( Math, currents );
		Imin = Math.min.apply( Math, currents );
		
		for(var i=0; i<currents.length; i++) {
			if (currents[i] > min) bigger = true;
		}
		
		if ( (Imax-Imin)/Imax > val && bigger ) {
			_phaseStartFlag = true;
			ABB.Simulator.Global.isStart = true;								
			if( !ABB.Simulator.Global.breakerOpen ) {
				objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );									
			}
			waitIfTrip(index, -1, -1);
		}
		else {
			if (_phaseStartFlag == true && ABB.Simulator.Global.isAlarm == false) {
				_phaseStartFlag = false;
				ABB.Simulator.Global.isStart = false;
				objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
				objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
				shouldLedsBeOff();
			}				
		}				
	}
	
	function checkNegativeStart( index, no) {
		var val = ABB.Simulator.UI.CardMode.getValueNode(_signalTreshold[index]);
		var ngSeI;	
		
		ngSeI = Math.sqrt( 1/9*Math.pow( (_injectedValue[0] - 0.5*(_injectedValue[1] + _injectedValue[2])), 2 ) + 1/12*Math.pow( (_injectedValue[1] - _injectedValue[2]), 2 ) );
		
		if ( ngSeI > (val*_nominalCurrent) ) {
			_negativeStartFlag[no] = true;
			ABB.Simulator.Global.isStart = true;								
			if( !ABB.Simulator.Global.breakerOpen ) {
				objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );									
			}
			waitIfTrip(index, -1, no);
		}
		else {
			if (_negativeStartFlag[no] == true && ABB.Simulator.Global.isAlarm == false) {
				_negativeStartFlag[no] = false;
				ABB.Simulator.Global.isStart = false;
				objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
				objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
				shouldLedsBeOff();
			}	
		}	
	}
		
	/**
	* Starts a timeout sequence before calling checkTripCondition.
	* This simulates the maximum delay for a signal before trip condition is met.
	* Parameters:
	* index			Index from _signalIds array to get correct node id
	*/
	function waitIfTrip( index, currentID, overcurrent ) {
		// get protect stage timeout for trip
		var val = ABB.Simulator.UI.CardMode.getValueNode(_delayTime[index]);
		// convert requested value from seconds to milliseconds
		val = val * 1000;
		if ( currentID == -1 ) {
			if ( overcurrent == -1 ) {
				setTimeout('ABB.Simulator.Global.objErrorSimulation.checkPhaseTrip('+ index +')', val);
			}
			else
				setTimeout('ABB.Simulator.Global.objErrorSimulation.checkNegativeTrip('+ index +', '+ overcurrent +')', val);
		}
		else {
			setTimeout('ABB.Simulator.Global.objErrorSimulation.checkTripCondition('+ index +', '+ currentID +', '+ overcurrent +')', val);
		}
	}
	
	function getFormattedDate() {
		var currentTime = new Date();
		var month = currentTime.getMonth() + 1;
		if (month < 10) month = "0" + month;
		var day = currentTime.getDate();
		if (day < 10) day = "0" + day;
		var year = currentTime.getFullYear();
		var formattedDate = day + "." + month + "." +year;
		return formattedDate;
	}
	
	function getFormattedTime() {
		var currentTime = new Date();
		var milliseconds = currentTime.getTime();
		milliseconds = milliseconds.toString();
		milliseconds = milliseconds.substring(milliseconds.length-3)
		var hours = currentTime.getHours()
		if (hours < 10) hours = "0" + hours;
		var minutes = currentTime.getMinutes()
		if (minutes < 10) minutes = "0" + minutes;
		var seconds = currentTime.getSeconds();
		if (minutes < 10) seconds = "0" + seconds;
		var formattedTime = hours + ":" + minutes + ":" + seconds  + "." + milliseconds
		return formattedTime;
	}
	
	/**
	* Checks if Arc protection fault is hot
	*/
	function checkArcProtectionFault(index, locks) {	
		var oSLD = ABB.Simulator.Global.GetSingleLineDiagram();
		var text = 'ARCSARC' + (index+1);

		var mode= ABB.Simulator.UI.CardMode.getValueNode(_arcMode[index])*1;
		
		switch(mode) {
			case 0:		//Light & current	
				var count = 0;
				for(var i=0; i<4; i++) {
					if (!locks[i]) {
					count++;
					}
				}
				if(count > 0) {
					// get Arc Light Protect stages and convert them to Number
					var arci = ABB.Simulator.UI.CardMode.getValueNode(_arcTreshold[index][0]);
					var arcio = ABB.Simulator.UI.CardMode.getValueNode(_arcTreshold[index][1]);
					arci = arci*_nominalCurrent;
					arcio = arcio*_nominalEarthCurrent;
							
					if( (locks[0] && _injectedValue[0] > arci) || (locks[1] && _injectedValue[1] > arci) || (locks[2] && _injectedValue[2] > arci ) || (locks[3] && _injectedValue[3] > arcio) ) {
						_arcFlag[index] = true;
						ABB.Simulator.Global.isStart = true;			
						if( !ABB.Simulator.Global.breakerOpen ) {
							objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );				
						}
						else {
							if(_arcFlag[index] == true && ABB.Simulator.Global.isAlarm == false) {
								_arcFlag[index] = false
								ABB.Simulator.Global.isStart = false;
								objLEDs.SetLedReadyState( ABB.Simulator.Global.Constants.LEDStateGreen );
								objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateInactive );
								shouldLedsBeOff();
							}
						}
					}
				}
				break;
			case 1:		//Light only
				_arcFlag[index] = true;
				ABB.Simulator.Global.isStart = true;			
				if( !ABB.Simulator.Global.breakerOpen ) {
					objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );				
				}
				break;
			case 2: //bi controlled
				break;
		}
			
		if( _arcFlag[index] ) {
			_arcTripFlag[index] = true;					
			if( !ABB.Simulator.Global.breakerOpen ) {
				if(!ABB.Simulator.Global.isAlarm) {
					var time = getFormattedTime();
					var date = getFormattedDate();
					ABB.Simulator.UI.Dialog.AlertDialog( "OPERATE<br/>" + date+ "<br/>" + time, text );
					ABB.Simulator.UI.Dialog.ShowDialog();
					addEvent(date + "<br/>" + time + "<br/>" + text + "<br/>Operate");
					objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );
					objLEDs.SetLedTripState( ABB.Simulator.Global.Constants.LEDStateRed );
					ABB.Simulator.Global.isAlarm = true;
				
					//if fault simulation panel is open, open the breaker
					if(oSLD) { 					
						oSLD.OpenBreaker();
					}
					else {
						ABB.Simulator.Global.breakerOpen = true;
					}
				}
			}				
		}		
	}	
	
	/* Public Functions */
	/**
	* Checks if signals trip condition is met.
	* Parameters:
	* index			Index from _signalIds array to get correct node id
	*/
	this.checkTripCondition = function ( index, currentID, overcurrent ) {
		var oSLD = ABB.Simulator.Global.GetSingleLineDiagram();
		var _startCounter = 0;
		var text = _protectStages[index] + ' ';
		
		if (overcurrent) {
			for (var i=0; i<_overcurrentStartFlags[index].length; i++) {
				if ( _overcurrentStartFlags[index][i] == true ) {
					_startCounter += 1;
				}
			}
			var startNo = ABB.Simulator.UI.CardMode.getValueNode(_NoStartPh[index])*1;
			switch (startNo) {
				case 0:
						if( _overcurrentStartFlags[index][currentID] == true ) {							
							_overcurrentTripFlags[index][currentID] = true;							
							text += _inputTag[currentID];		//alarm message text
						}
						else {
							_overcurrentTripFlags[index][currentID] = false;
						}
						break;
				case 1:
						if( _overcurrentStartFlags[index][currentID] == true && _startCounter == 2 ) {
							_overcurrentTripFlags[index][currentID] = true;							
							for (var i=0; i<_overcurrentStartFlags[index].length; i++) {
								if ( _overcurrentStartFlags[index][i] == true ) {
									text += _inputTag[i] + ' ';
								}
							}
						}
						else {
							_overcurrentTripFlags[index][currentID] = false;
						}
						break;
				case 2:
						if( _overcurrentStartFlags[index][currentID] == true && _startCounter == 3 ) {
							_overcurrentTripFlags[index][currentID] = true;							
							for (var i=0; i<_overcurrentStartFlags[index].length; i++) {
								if ( _overcurrentStartFlags[index][i] == true ) {
									text += _inputTag[i] + ' ';
								}
							}
						}
						else {
							_overcurrentTripFlags[index][currentID] = false;
						}
						break;						
			}
			if( !ABB.Simulator.Global.isAlarm && _overcurrentTripFlags[index][currentID] ) {
				if(!ABB.Simulator.Global.breakerOpen) {					
					var time = getFormattedTime();
					var date = getFormattedDate();
					ABB.Simulator.UI.Dialog.AlertDialog( "OPERATE<br/>" + date+ "<br/>" + time, text );
					ABB.Simulator.UI.Dialog.ShowDialog();
					addEvent(date + "<br/>" + time + "<br/>" + text + "<br/>Operate");
					objLEDs.SetLedTripState( ABB.Simulator.Global.Constants.LEDStateRed );
					objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );
					objLEDs.SetLedProgrammableState(0, ABB.Simulator.Global.Constants.LEDStateRed );
					ABB.Simulator.Global.isAlarm = true;							
					
					//if fault simulation panel is open, open the breaker
					if(oSLD) {
						oSLD.OpenBreaker();
					}
					else {
						ABB.Simulator.Global.breakerOpen = true;
					}
				}
			}	
		}
		else {
			if( _earthFaultStartFlags[index-4] == true) {
				_earthFaultTripFlags[index-4] = true;				
			}
			else {
				_earthFaultTripFlags[index-4] = false;
			}
			if( !ABB.Simulator.Global.isAlarm && _earthFaultTripFlags[index-4] ) {
				if(!ABB.Simulator.Global.breakerOpen) {					
					var time = getFormattedTime();
					var date = getFormattedDate();
					ABB.Simulator.UI.Dialog.AlertDialog( "OPERATE<br/>" + date+ "<br/>" + time, text );
					ABB.Simulator.UI.Dialog.ShowDialog();
					addEvent(date + "<br/>" + time + "<br/>" + text + "<br/>Operate");
					objLEDs.SetLedTripState( ABB.Simulator.Global.Constants.LEDStateRed );
					objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );
					var check= index-4;
					switch (check) {
						case 0:
							objLEDs.SetLedProgrammableState(2, ABB.Simulator.Global.Constants.LEDStateRed );
							break;
						case 1:
						case 2:
						case 3:
							objLEDs.SetLedProgrammableState(1, ABB.Simulator.Global.Constants.LEDStateRed );
							break;						
					}					
					ABB.Simulator.Global.isAlarm = true;							
										
					//if fault simulation panel is open, open the breaker
					if(oSLD) {
						oSLD.OpenBreaker();
					}
					else {
						ABB.Simulator.Global.breakerOpen = true;
					}
				}
			}	
		}			
	}
	
	this.checkPhaseTrip = function(index) {
		var oSLD = ABB.Simulator.Global.GetSingleLineDiagram();		
		var text = _protectStages[index] + ' ';		
			
			if( _phaseStartFlag ) {
				_phaseTripFlag = true;
			}
			else {
				_phaseTripFlag = false;
			}
		
		if( !ABB.Simulator.Global.isAlarm && _phaseTripFlag ) {
			if(!ABB.Simulator.Global.breakerOpen) {					
				var time = getFormattedTime();
				var date = getFormattedDate();
				ABB.Simulator.UI.Dialog.AlertDialog( "OPERATE<br/>" + date+ "<br/>" + time, text );
				ABB.Simulator.UI.Dialog.ShowDialog();
				addEvent(date + "<br/>" + time + "<br/>" + text + "<br/>Operate");
				objLEDs.SetLedTripState( ABB.Simulator.Global.Constants.LEDStateRed );
				objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );
				objLEDs.SetLedProgrammableState(3, ABB.Simulator.Global.Constants.LEDStateRed );
				ABB.Simulator.Global.isAlarm = true;				
				
				//if fault simulation panel is open, open the breaker
				if(oSLD) {
					oSLD.OpenBreaker();
				}
				else {
					ABB.Simulator.Global.breakerOpen = true;
				}
			}
		}
	}

	this.checkNegativeTrip = function(index, no) {
		var oSLD = ABB.Simulator.Global.GetSingleLineDiagram();		
		var text = _protectStages[index] + ' ';		
			
			if( _negativeStartFlag[no] ) {
				_negativeTripFlag[no] = true;
			}
			else {
				_negativeTripFlag[no] = false;
			}
		
		if( !ABB.Simulator.Global.isAlarm && _negativeTripFlag[no] ) {
			if(!ABB.Simulator.Global.breakerOpen) {					
				var time = getFormattedTime();
				var date = getFormattedDate();
				ABB.Simulator.UI.Dialog.AlertDialog( "OPERATE<br/>" + date+ "<br/>" + time, text );
				ABB.Simulator.UI.Dialog.ShowDialog();
				addEvent(date + "<br/>" + time + "<br/>" + text + "<br/>Operate");
				objLEDs.SetLedTripState( ABB.Simulator.Global.Constants.LEDStateRed );
				objLEDs.SetLedStartState( ABB.Simulator.Global.Constants.LEDStateYellow );
				objLEDs.SetLedProgrammableState(3, ABB.Simulator.Global.Constants.LEDStateRed );
				ABB.Simulator.Global.isAlarm = true;							
				
				//if fault simulation panel is open, open the breaker
				if(oSLD) {
					oSLD.OpenBreaker();
				}
				else {
					ABB.Simulator.Global.breakerOpen = true;
				}
			}
		}
	
	}
		
	this.clearStartState = function() {
		for(var i=0; i<_overcurrentStartFlags.length; i++) {
			for(var j=0; j<_overcurrentStartFlags[i].length; j++) {
				_overcurrentStartFlags[i][j] = false;
			}
		}
		
		for(var i=0; i<_earthFaultStartFlags.length; i++) {
			_earthFaultStartFlags[i] = false;
		}
		
		for(var i=0; i<_negativeStartFlag.length; i++) {
			_negativeStartFlag[i] = false;
		}
		
		for(var i=0; i<_arcFlag.length; i++) {
			_arcFlag[i] = false;
		}
		
		_phaseStartFlag = false;		
		ABB.Simulator.Global.isStart = false;
	}
	
	this.clearTripFlags = function() {
		for(var i=0; i<_overcurrentTripFlags.length; i++) {
			for(var j=0; j<_overcurrentTripFlags[i].length; j++) {
				_overcurrentTripFlags[i][j] = false;
			}
		}
		
		for(var i=0; i<_earthFaultTripFlags.length; i++) {
			_earthFaultTripFlags[i] = false;
		}
		
		for(var i=0; i<_negativeTripFlag.length; i++) {
			_negativeTripFlag[i] = false;
		}
		
		for(var i=0; i<_arcTripFlag.length; i++) {
			_arcTripFlag[i] = false;
		}
		
		_phaseTripFlag = false;		
	}
			
	function addEvent( text ) {
		_eventList[_eventCounter] = text;
		_eventCounter++;
	}
	
	this.getEvent = function ( i ) {
		return _eventList[i];
	}
	
	this.getEventListLength = function () {
		var counter = 0;
		while(_eventList[counter] != null) {
			counter++;
		}
		return counter;
	}
	
	this.clearEventList = function () {
		_eventList = new Array(50);
		_eventCounter = 0;
	}
	
	this.SetInjectedValue = function( index, value ) {
		_injectedValue[index] = value;			
	}	
	
	this.GetInjectedValue = function(index) {
		return _injectedValue[index];
	}
	
	this.GetSequenceCurrent = function(index) {
		var current;
		
		switch(index) {
			case 0:
				current = Math.sqrt( 1/9*Math.pow( (_injectedValue[0] - 0.5*(_injectedValue[1] + _injectedValue[2])), 2 ) + 1/12*Math.pow( (_injectedValue[1] - _injectedValue[2]), 2 ) );
				break;
			case 1:
				current = 1/3*(_injectedValue[0] + _injectedValue[1] + _injectedValue[2]);
				break;
			case 2:
				current = Math.sqrt( 1/9*Math.pow( (_injectedValue[0] - 0.5*(_injectedValue[1] + _injectedValue[2])), 2 ) + 1/12*Math.pow( (_injectedValue[2] - _injectedValue[1]), 2 ) );
				break;
		}
		return current;
	}

	this.SetLightSensor = function(index) {
		if(_isLightSensorEnabled[index]) {
			_isLightSensorEnabled[index] = false;
		}
		else {
			_isLightSensorEnabled[index] = true;
		}
	}

	this.GetLightSensor = function(index) {
		return _isLightSensorEnabled[index];
	}
	
	/**
	* Checks for overcurrent and earth -faults 
	*/
	this.CheckForFaults = function() {
		simulateFaults();
	}	
}

