﻿/** 
 * @fileoverview This file is to be used for showing dialogs on
 * simulator screen. 
 *
 * @author jv@wapice.com
 * @version 0.1 
 *
 * Different Dialogtypes
 * 0 = Modal Dialog -  OK, YesNo, YesNoCancel Buttons
 * 1 = Login Dialog - Select user level 
 * 2 = Enter password dialog
 * 3 = Select Setting Group
 * 4 = Alert Screen - For Alert messages
 */

/**********************************************************************************
 * Modal Dialog - [ 0 ]
 **********************************************************************************/
 
/**
 * Function is called when HMI Enter button is pressed
 * @param text - text field in dialog
 * @param caption - headline
 * @param buttons - Definitions for buttons: OK, YesNo, YesNoChancel
 * @param defaultButton - Zero based value for first active button
 * @exception Dialog result is -1
 * @see next constructor supports type field
 */
ABB.Simulator.UI.Dialog.ModalDialog = function ( text, caption, buttons, defaultButton ) {
	this.dialogResult = -1;
	this.activeButton = defaultButton;
	this.text = text;
	this.caption = caption;
	this.buttons = buttons;
	this.dialogtext = "";
	this.dialogType = 0;
}

/**
 * Function is called when HMI Enter button is pressed
 * @param text - text field in dialog
 * @param caption - headline
 * @param buttons - Definitions for buttons: OK, YesNo, YesNoChancel
 * @param defaultButton - Zero based value for first active button
 * @param Save dialog type to identify it in buttonHandler class
 * @exception Dialog result is -1
 */
ABB.Simulator.UI.Dialog.ModalDialog = function ( text, caption, buttons, defaultButton, type ) {
	this.dialogResult = -1;
	this.activeButton = defaultButton;
	this.caption = caption;
	this.text = text;
	this.buttons = buttons;
	this.dialogtext = "";
	this.type = type;
	this.dialogType = 0;
}

/**********************************************************************************
 * Login Dialog - [ 1 ]
 **********************************************************************************/

/**
 * Function initializes login screen
 * @exception Dialog result is -1
 */
ABB.Simulator.UI.Dialog.LoginDialog = function ( ) {
	this.dialogResult = -1;
	this.selectedUser = 0;
	this.caption = "Login";
	this.text = "Selected User";
	this.dialogtext = "";
	this.dialogType = 1;
}

/**********************************************************************************
 * Enter Password Dialog - [ 2 ]
 **********************************************************************************/

/**
 * Function initializes login screen
 * @exception Dialog result is -1
 */
ABB.Simulator.UI.Dialog.SetPasswordDialog = function ( userLevel ) {
	this.dialogResult = -1;
	this.activeColumn = 0;
	switch( userLevel ) {
		case 1:
			this.caption = "VIEWER";
		break;
		case 2:
			this.caption = "OPERATOR";
		break;
		case 3:
			this.caption = "ENGINEER";
		break;
		case 4:
			this.caption = "ADMINISTRATOR";
		break;
		default:
			this.dialogResult = -1;
		break;
	}
	this.text = "Enter Password:<br/>";
	this.dialogtext = "";
	this.passWord = "0000";
	this.dialogType = 2;
}

/**********************************************************************************
 * Select Setting Group Dialog - [ 3 ]
 **********************************************************************************/

/**
 * Function initializes Setting Group selection
 * @exception Dialog result is -1
 */
ABB.Simulator.UI.Dialog.SelectSGDialog = function (  ) {
	this.dialogResult = -1;
	this.activeColumn = 0;
	this.caption = ABB.Simulator.UI.CardMode.getValueNode(ABB.Simulator.Global.Constants.BayNameId);
	this.text = "";
	this.dialogtext = "";
	this.dialogType = 3;
}	
/**********************************************************************************
 * Alert Dialog - [ 4 ]
 **********************************************************************************/

/**
 * Function shows alert on screen
 * @exception Dialog result is -1
 */
ABB.Simulator.UI.Dialog.AlertDialog = function ( text, caption ) {
	this.dialogResult = -1;
	this.caption = caption;
	this.text = text;
	this.dialogtext = "";
	this.dialogType = 4;
	this.savedText = text;
	this.savedCaption = caption;
}

/**********************************************************************************
 * Common Functions
 **********************************************************************************/

/**
 * Function Updates dialog text on screen
 */
ABB.Simulator.UI.Dialog.ShowDialog = function () {	
	this.dialogtext = "";
		
	ABB.Simulator.Global.showDialog = true;
	
	clearTimeout(ABB.Simulator.Global.timeoutMonitor);	
	
	this.dialogtext += this.caption + "<br/>"+"<br/>";	
	this.dialogtext += this.text;
	
	switch( this.dialogType ) {
		case 0:	// Modal Dialog
			this.dialogtext += "<br/><br/>";
			switch( this.buttons ) {
				case "OK":
					this.dialogtext += "      <b>OK</b><br/>";
					this.dialogResult = "Ok"
				break;
				case "YesNo":
					if (this.activeButton == 0) {
						this.dialogtext += "     <b>Yes</b> No<br/>";
						this.dialogResult = "Yes"
					}
					else if (this.activeButton == 1) {
						this.dialogtext += "     Yes <b>No</b><br/>";
						this.dialogResult = "No";
					}
				break;
				case "YesNoCancel":
					if (this.activeButton == 0) {
						this.dialogtext += "   <b>Yes</b> No Cancel<br/>";
						this.dialogResult = "Yes"
					}
					else if (this.activeButton == 1) {
						this.dialogtext += "   Yes <b>No</b> Cancel<br/>";
						dialogResult = "No";
					}
					else if (this.activeButton == 2) {
						this.dialogtext += "   Yes No <b>Cancel</b><br/>";
						this.dialogResult = "Cancel";
					}
				break;
				default:
					this.dialogresult = -1;
				break;
			}
		break;
		case 1: //Login Dialog 
			this.dialogtext += "<br/><br/>";
			switch( this.selectedUser ) {
				case 0:
					this.dialogtext += "=<b>VIEWER</b><br/>";
					this.dialogResult = 1;
				break;
				case 1:
					this.dialogtext += "=<b>OPERATOR</b><br/>";
					this.dialogResult = 2;
				break;
				case 2:
					this.dialogtext += "=<b>ENGINEER</b><br/>";
					this.dialogResult = 3;
				break;
				case 3:
					this.dialogtext += "=<b>ADMINISTRATOR</b><br/>";
					this.dialogResult = 4;
				break;
				default:
					this.dialogResult = -1;
				break;
			}
		break;
		case 2:	//Enter password Dialog
			this.dialogtext += "<br/><br/>";
			var temp = this.passWord.charAt(3 - this.activeColumn);
			switch( this.activeColumn ) {
				case 0:
					this.dialogtext += "=***<b>" + temp + "</b><br/>";
					this.dialogResult = this.passWord;
				break;
				case 1:
					this.dialogtext += "=**<b>" + temp + "</b>*<br/>";
					this.dialogResult = this.passWord;
				break;
				case 2:
					this.dialogtext += "=*<b>" + temp + "</b>**<br/>";
					this.dialogResult = this.passWord;
				break;
				case 3:
					this.dialogtext += "=<b>" + temp + "</b>***<br/>";
					this.dialogResult = this.passWord;
				break;
				default:
					this.dialogResult = -1;
				break;
			}
		break;
		case 3:	//Select Setting Group Dialog
			this.dialogtext +=  "Edit Group:↕" + ABB.Simulator.Global.editGroupValue	+ "<br/><br/>";
			this.dialogtext += "Active group=" + ABB.Simulator.Global.activeSettingGroup + "<br/>";
		break;
		case 4:	//Alert Dialog
		break;
	}
	/**
	 *Align text center
	 */
	var rowText = this.dialogtext.split('<br/>');
	this.dialogtext = "";			
	for (i=1;i<rowText.length;i++) {
		if ( rowText[i].length > 0 ) {
			var diff = 20 - rowText[i].length;
			for (j=1; j<(diff-Math.round(diff/2)); j++) { 
				 rowText[i] = " " + rowText[i];							// Add spaces 
			}
		}
	}
	for (i=0;i<rowText.length;i++) {
			 this.dialogtext += rowText[i] + "<br/>";							// Add spaces 
	}
	ABB.Simulator.Global.display.refresh( this.dialogtext );
}

/**
 * Function sets single character in string at pointed location
 * @param str - TARGET STRING	 
 * @param index - CHARACTER INDEX IN TARGET STRING
 * @param char - character that replaces old one
 * @return new string
 */
function setCharAt(str,index,char) {
	if(index > str.length-1) return str;
	return str.substr(0,index) + char + str.substr(index+1);
}

/**
 * Function returns dialog result and closes dialog
 * @return dialogResult - Selectyed value in dialog  OK, Yes, No, Chancel || Selected User level || Entered password
 * @exception On Error returns -1
 */
ABB.Simulator.UI.Dialog.GetDialogResult = function ( ) {
	ABB.Simulator.Global.showDialog = false;
	return this.dialogResult;
}

/**
 * Function returns dialog result and closes dialog
 * @return dialogResult - Selectyed value in dialog  OK, Yes, No, Chancel || Selected User level || Entered password
 * @exception On Error returns -1
 */
ABB.Simulator.UI.Dialog.ShowAlertDialog = function ( ) {
	this.dialogType = 4;
	this.text = this.savedText;
	this.caption = this.savedCaption;
	ABB.Simulator.UI.Dialog.ShowDialog();
}

/**********************************************************************************
 * Button Handlers
 **********************************************************************************/

/**
 * Called when left arrow button has pressed
 */
ABB.Simulator.UI.Dialog.ButtonLeft = function ( ) {
	switch( this.dialogType ) {
		case 0:	// Modal Dialog
			if (this.activeButton > 0) {
				this.activeButton--;
			}
		break;
		case 1: //Login Dialog 
		break;
		case 2:	//Enter password Dialog
			if (this.activeColumn < 3) {
				this.activeColumn++;
			}
		break;
		case 3:	//Select Setting Group Dialog
		break;
		case 4:	//Alert Dialog
		break;
	}
	ABB.Simulator.UI.Dialog.ShowDialog();
}

/**
 * Called when right arrow button has pressed
 */
ABB.Simulator.UI.Dialog.ButtonRight = function ( ) {
	switch( this.dialogType ) {
		case 0:	// Modal Dialog
			switch( this.buttons ) {
				case "OK":
					this.activeButton = 0;
				break;
				case "YesNo":
					if (this.activeButton < 1) {
						this.activeButton++;
					}
				break;
				case "YesNoCancel":
					if (this.activeButton < 2) {
						this.activeButton++;
					}
				break;
			}
		break;
		case 1: //Login Dialog 
		break;
		case 2:	//Enter password Dialog
			if (this.activeColumn > 0) {
				this.activeColumn--;
			}
		break;
		case 3:	//Select Setting Group Dialog
		break;
		case 4:	//Alert Dialog
		break;
	}
	ABB.Simulator.UI.Dialog.ShowDialog();
}

/**
 * Called when up arrow button has pressed
 */
ABB.Simulator.UI.Dialog.ButtonUp = function ( ) {
	switch( this.dialogType ) {
		case 0:	// Modal Dialog
		break;
		case 1: //Login Dialog 
			if ( this.selectedUser > 0 ) {
				this.selectedUser--;
			}
		break;
		case 2:	//Enter password Dialog
			var temp = this.passWord.charAt(3 - this.activeColumn);
			if (temp < 9) {
				temp++;
			}
			else {
				temp = 0;
			}
			this.passWord = setCharAt(this.passWord , (3 - this.activeColumn) ,temp);
		break;
		case 3:	//Select Setting Group Dialog
			if ( ABB.Simulator.Global.editGroupValue < 4 ) {
				ABB.Simulator.Global.editGroupValue++;
			}
		break;
		case 4:	//Alert Dialog
		break;
	}
	ABB.Simulator.UI.Dialog.ShowDialog();
}

/**
 * Called when down arrow button has pressed
 */
ABB.Simulator.UI.Dialog.ButtonDown = function ( ) {
	switch( this.dialogType ) {
		case 0:	// Modal Dialog
		break;
		case 1: //Login Dialog
			if ( this.selectedUser < 3 ) {
				this.selectedUser++;
			}
		break;
		case 2:	//Enter password Dialog
			var temp = this.passWord.charAt(3 - this.activeColumn);
			if (temp > 0) {
				temp--;
			}
			else {
				temp = 9;
			}
			this.passWord = setCharAt(this.passWord , (3 - this.activeColumn) ,temp);
		break;
		case 3:	//Select Setting Group Dialog
			if ( ABB.Simulator.Global.editGroupValue > 1 ) {
				ABB.Simulator.Global.editGroupValue--;
			}
		break;
		case 4:	//Alert Dialog
		break;
	}
	ABB.Simulator.UI.Dialog.ShowDialog();
}
