﻿/** 
 * @fileoverview This file implements the functionality of
 * REF615 Simulator LCD display. Display size is defined in global
 * constants file
 *
 * @author agotukas@wapice.com
 * @author jv@wapice.com
 * @version 0.3
 */
 

ABB.Simulator.UI.Display.DisplayContent = function () {
	this.text;
	this.link;
	this.valueid;
}

/**
 * Function is called when HMI Enter button is pressed
 * @param PARAMETERNAME - PARAMETER DESCRIPTION (as many @param tags as needed)
 * @return what is the method returning
 * @exception TYPEWHEN THROWN
 * @see what else methods should be looked at
 */
ABB.Simulator.UI.Display.DisplayInput = function () {
    this.valueid;
    this.type;
    this.min;
    this.max;
    this.value;
    this.link;
    this.title;
    this.format;
    this.unit;
    this.options	= new Array();
    this.groups		= new Array();
    this.activeoption;
	this.access;
	this.category;
}

/**
 * Function is called when HMI Enter button is pressed
 * @param PARAMETERNAME - PARAMETER DESCRIPTION (as many @param tags as needed)
 * @return what is the method returning
 * @exception TYPEWHEN THROWN
 * @see what else methods should be looked at
 */
ABB.Simulator.UI.Display.DisplayCard = function () {
	this.id;
	this.headline;
	this.mode;
	this.content	= new Array();
	this.help;
	this.text;
}

/**
 * Saves page to page histery when new one is loaded
 * @param id - Id of the page to be saved
 */
ABB.Simulator.UI.Display.PutPageHistory = function ( id ) {
	var ph = new ABB.Simulator.UI.Display.DisplayHistory();
    ph.id = id;
    ph.dspposition = ABB.Simulator.Global.dspposition;         
    ABB.Simulator.Global.PAGEHISTORY[ABB.Simulator.Global.PAGEHISTORY.length] = ph;
}

/**
 * Returns previous page from page history and sets display position
 * @return id of previous page
 */
ABB.Simulator.UI.Display.GetPageHistory = function () {
	var id = "";
	if( ABB.Simulator.Global.PAGEHISTORY.length > 0 ) {
		var ph = ABB.Simulator.Global.PAGEHISTORY.pop();
		id = ph.id;
		ABB.Simulator.Global.dspposition = ph.dspposition;
	}
	return id;
}

/**
 * Clears whole page history
 */
ABB.Simulator.UI.Display.ClearPageHistory = function () {
		ABB.Simulator.Global.PAGEHISTORY.length = 0;
}

/**
 * Constructor for display history object
 * @see #ABB.Simulator.UI.Display.ClearPageHistory
 * @see #ABB.Simulator.UI.Display.GetPageHistory
 * @see #ABB.Simulator.UI.Display.PutPageHistory
 */
ABB.Simulator.UI.Display.DisplayHistory = function () {
	this.id;
	this.dspposition;
	this.cursorpos;
}

/**
 * Refreshs display when dialog is not open
 */
ABB.Simulator.UI.Display.RefreshDisplay = function () {
	if ( !ABB.Simulator.Global.showDialog ) {
		ABB.Simulator.UI.CardMode.DrawCardInMode();
	}
}

/**
 * Construct a new LCDDisplay object.
 * @class This is the basic LCDDisplay class.  
 * @constructor
 */
ABB.Simulator.UI.Display.LCDDisplay = function () {
	var _objDisplay;
	var _objDisplayCell;
	
	this.screenpointer;
	
	/**
	 * Sets display object
	 * @param objDisplay - Display object
	 */
	this.setDisplay = function ( objDisplay ) {
		_objDisplay = objDisplay;
	}
	
	/**
	 * Clears display content
	 */
	this.clearDisplay = function () {
		screenpointer = 0;
		_objDisplay.innerHTML = "";
	}	
	
	/**
	 * Function converts text row to HTML <img> tags
	 * @param rowText - Text to be converted
	 * @param background - active/regular to set normal or inversed font color
	 * @return Text row converted to image tags
	 */
	function convertTextToImage (rowText, background) {
		var regularPath = 'ied_images/Fonts/';
		var highlightPath = 'ied_images/Fonts_active/';
		var path;
		var returnRow = '';	
		var charcodes = '';
		
		if (background == 'regular') {
			path = regularPath;
		}
		if (background == 'active') {
			path = highlightPath;
		}		
			
		if ( rowText.length > 0) {
			for (var j=0; j<rowText.length; j++) {					
					
				var decCode = rowText.charCodeAt(j);				
				var hexCode = (decCode.toString(16)).toUpperCase();							
				switch (hexCode.length) {
					case 2:
						returnRow += '<img src = "' + path + '00'+ hexCode + '.gif">';
					break;
					case 3:
						returnRow += '<img src = "' + path + '0'+ hexCode + '.gif">';
					break;
					case 4:
						returnRow += '<img src = "' + path + hexCode + '.gif">';
					break;
				}												
			}
		}
		return returnRow;	
	}
	
	/**
	 * Refreshes display content
	 * @param text - display content -  Handles following tags: <br/> = line break, <b></b> uses inversed fonts
	 * @see convertTextToImage - converts text to html image tags
	 */
	this.refresh = function( text ) {	
					
		var columns = ABB.Simulator.Global.Constants.DisplayColumns;
		var rows = 5;
		var scrollPositions = 4;			
		var rowBitm = new Array();
		var screen = '';
		var separator = '<br/>';
		var noEndTag = false;
		var index;
		var head = '';
		
		var rowText = text.split('<br/>'); 								// gives extra row with '' value at the end of array
									
		//split text to image sequence
		for (var i=0; i<rowText.length; i++) {				
			rowBitm[i] = '';
			index = rowText[i].search('<b>'); 							//find the tag							
			if (index != -1) {											//if tag is found:		
				if (index != 0) {
					noEndTag = true;
					var newRowPiece = rowText[i].substring(0,index); 	//cut first part before tag						
					if(convertTextToImage(newRowPiece, 'regular') != '') {
						rowBitm[i] += convertTextToImage(newRowPiece, 'regular');				
					}						
				}
				else {
					var newRowPiece = '';
				}
				var temp = rowText[i].substr(index+3); 					//save rest part of string after tag
				index = temp.search('</b>'); 							//search rest of the string for end tag
				if (index != -1) { 										//if tag was found:
					noEndTag = false;
					newRowPiece = temp.substring(0,index);				// newRow holds now second part of the string (piece between tags)
					if(convertTextToImage(newRowPiece, 'active') != '') {
						rowBitm[i] += convertTextToImage(newRowPiece, 'active');				
					}	
					if(temp.substr(index+4)!='') { 						//if there's anything after end tag left
						newRowPiece = temp.substr(index+4); 			//add to the newRow
						if(convertTextToImage(newRowPiece, 'regular') != '') {
							rowBitm[i] += convertTextToImage(newRowPiece, 'regular');				
						}
					}							
				}
				else { 													//if there is row without closing <b> tag 
					newRowPiece = temp;
					if (newRowPiece.length > ABB.Simulator.Global.Constants.DisplayColumns) {
						newRowPiece = newRowPiece.substring(0,ABB.Simulator.Global.Constants.DisplayColumns);
					}
					if(convertTextToImage(newRowPiece, 'active') != '') {
						rowBitm[i] += convertTextToImage(newRowPiece, 'active');				
					}
					var diff = columns - newRowPiece.length;
					if (noEndTag) diff--;
					if ( diff > 0 ) {
						for (var k=0; k<diff; k++) {
							rowBitm[i] += '<img src = "ied_images/Fonts_active/0020.gif">';
						}
					}	
				}					
			}
			else { 
				if (rowText[i].length >= columns) {						//Cut row if it is too long
					rowText[i] = rowText[i].substring(0, columns);
				}
				var diff = columns - rowText[i].length;					
				if ( i == 0 ) {											//If row is headline
					for (var j=0; j<Math.round(diff/2)-1; j++) {
						head += " "; 
					}
					head += rowText[i];
					
					for (j=0; j<(diff-Math.round(diff/2)); j++) { 
						head += " ";							// Add spaces 
					}
					if (head.length == columns) 
						head = head.substring(0, columns-1);
					rowText[i] = head;
					switch( ABB.Simulator.Global.userLevel ) {
						case 1:										//Viewer
							rowText[i] += "V";
						break;
						case 2:										//Operator
							rowText[i] += "O";
						break;
						case 3:										//Engineer		
							rowText[i] += "E";
						break;
						case 4:										//Administrator						
							rowText[i] += "A";
						break;
						default:
							rowText[i] += "";
						break;
					}						
				}
				rowBitm[i] = convertTextToImage(rowText[i], 'regular');
				
			}						
			rowBitm[i] += '<br/>';
			
		}			
		screen += '<div style="position:absolute;top:2px;left:0px;margin-top:10px;"><img src="ied_images/black_line.gif" width="128" height="1" alt="" /></div>';
		
		if ( !ABB.Simulator.Global.showDialog ) {			
			if (ABB.Simulator.Global.card.mode == "MenuNode" && ABB.Simulator.Global.card.content.length > 9) {				
				if ( ABB.Simulator.Global.dspposition < 4 )
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider1.gif" width="8" height="113" alt="" /></div>';
				else if (ABB.Simulator.Global.dspposition < Math.round((ABB.Simulator.Global.card.content.length / 2))+1 )
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider2.gif" width="8" height="113" alt="" /></div>';
				else if (ABB.Simulator.Global.dspposition < ABB.Simulator.Global.card.content.length - 1) 
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider3.gif" width="8" height="113" alt="" /></div>';
				else 
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider4.gif" width="8" height="113" alt="" /></div>'	
			}
			else if ((ABB.Simulator.Global.card.mode == "GroupMode" || (ABB.Simulator.Global.card.mode == "EditMode" && ABB.Simulator.Global.selectedValue.type != "sgnumeric") && (ABB.Simulator.Global.card.mode == "EditMode" && ABB.Simulator.Global.selectedValue.type != "sgoption") )  && ABB.Simulator.Global.card.content.length > 4) {
				if ( ABB.Simulator.Global.dspposition < 2 )
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider1.gif" width="8" height="113" alt="" /></div>';
				else if (ABB.Simulator.Global.dspposition < Math.round((ABB.Simulator.Global.card.content.length / 2)) )
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider2.gif" width="8" height="113" alt="" /></div>';
				else if (ABB.Simulator.Global.dspposition < ABB.Simulator.Global.card.content.length-1) 
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider3.gif" width="8" height="113" alt="" /></div>';
				else 
					screen += '<div class="scrollbar"><img src="ied_images/scrollbar_slider4.gif" width="8" height="113" alt="" /></div>'
			}
		}
		
		for (var i=0; i<rowBitm.length; i++) {			
			screen += rowBitm[i];				
		}
								
		_objDisplay.innerHTML = screen;				
	}
}